#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const meow = require("meow");
const opn = require("opn");
const path = require("path");
const update_notifier_1 = require("update-notifier");
const logger_1 = require("./logger");
const pkg = require("../package.json");
new update_notifier_1.UpdateNotifier({ pkg }).notify();
const generator_1 = require("./generator");
const server_1 = require("./server");
const copyFiles_1 = require("./build/copyFiles");
const exportPage_1 = require("./exportPage");
const parser_1 = require("./parser");
const cli = meow(`
  Usage:
    $ rocu dirname

  Options:
    --out-dir, -d   Output directory
    --dev, -D       Start development server
    --port, -p      Set port for development server
    --open, -o      Open development server in default browser
    --layout        Layout File Path
`, {
    flags: parser_1.flags,
});
const { develop: developOptions, build: buildOptions } = parser_1.parser(cli);
logger_1.notifyLog("rocu");
const main = async () => {
    if (developOptions) {
        logger_1.notifyLog("starting dev server");
        try {
            const srv = await server_1.server(developOptions.source, developOptions);
            const address = srv.address();
            let url;
            if (typeof address === "string") {
                logger_1.notifyLog(`listening on ${address}`);
                url = address;
            }
            else {
                const { port } = address;
                logger_1.notifyLog(`listening on port: ${port}`);
                url = path.join(`http://localhost:${port}`, developOptions.serverBasePath);
            }
            if (developOptions.open) {
                opn(url);
            }
        }
        catch (err) {
            logger_1.notifyLog("error", err);
            process.exit(1);
        }
    }
    if (buildOptions) {
        const pages = await generator_1.generateStaticPages(buildOptions.source, buildOptions);
        if (pages) {
            Promise.all([exportPage_1.exportPages(pages, buildOptions), copyFiles_1.copyAssetFiles(buildOptions)]);
        }
    }
};
main();
