"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const matter = require("gray-matter");
const path = require("path");
const recursive = require("recursive-readdir");
const helpers_1 = require("./helpers");
exports.isStartWithHttp = (url) => url.match(/^https?\:\/\/|^\/\//) !== null;
const rewriteScriptSource = (attribute, basePath) => {
    if (typeof attribute === "string") {
        return exports.isStartWithHttp(attribute) ? attribute : path.join(basePath, attribute);
    }
    const src = attribute.src ? (exports.isStartWithHttp(attribute.src) ? attribute.src : path.join(basePath, attribute.src)) : undefined;
    return { ...attribute, src };
};
const rewriteLinkSource = (attribute, basePath) => {
    if (typeof attribute === "string") {
        return exports.isStartWithHttp(attribute) ? attribute : path.join(basePath, attribute);
    }
    const href = attribute.href ? (exports.isStartWithHttp(attribute.href) ? attribute.href : path.join(basePath, attribute.href)) : undefined;
    return { ...attribute, href };
};
const rewriteMetaData = (globalSetting, localSetting, uri, option) => {
    const globalLinks = [...(globalSetting.link ? globalSetting.link : []), ...(globalSetting.css ? globalSetting.css : [])];
    const localLinks = [...(localSetting.link ? localSetting.link : []), ...(localSetting.css ? localSetting.css : [])];
    const rewriteLocalScripts = localSetting.js ? localSetting.js.map(src => rewriteScriptSource(src, uri)) : localSetting.js;
    const rewriteLocalLinks = localLinks.map(attribute => rewriteLinkSource(attribute, uri));
    return {
        ...globalSetting,
        ...localSetting,
        globalScripts: globalSetting.js ? globalSetting.js.map(js => rewriteScriptSource(js, option.serverBasePath)) : globalSetting.js,
        localScripts: rewriteLocalScripts,
        globalLinks: globalLinks.map(attribute => rewriteLinkSource(attribute, option.serverBasePath)),
        localLinks: rewriteLocalLinks,
    };
};
const formatUri = (uri, option) => {
    return path.join(option.serverBasePath, uri).replace(/\/index$/, "");
};
const getPage = (dirname, option) => async (filename) => {
    const globalSetting = helpers_1.getDefaultConfig(option.source).global || {};
    const ext = path.extname(filename);
    const relativePath = path.relative(dirname, filename);
    const uri = relativePath.slice(0, relativePath.length - ext.length);
    const fUri = formatUri(uri, option);
    const raw = fs.readFileSync(filename, "utf8");
    const { data, content } = matter(raw);
    const metaData = rewriteMetaData(globalSetting, data, path.dirname(fUri), option);
    return {
        uri: fUri,
        content,
        metaData,
        ext,
        filename,
        name: uri,
        raw,
    };
};
exports.getData = async (dirname, options) => {
    const allFiles = await recursive(dirname);
    const filenames = allFiles.filter(name => !/^\./.test(name));
    const jsxFilenames = filenames.filter(name => /\.jsx$/.test(name));
    const mdFilenames = filenames.filter(name => /\.mdx?/.test(name));
    const contentFiles = [...jsxFilenames, ...mdFilenames];
    const promises = contentFiles.map(getPage(dirname, options));
    const pages = await Promise.all(promises);
    return {
        dirname,
        pages,
    };
};
