"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const createTemplate_1 = require("./createTemplate");
const generateProps_1 = require("./generateProps");
const importer_1 = require("./importer");
const transformer_1 = require("./transformer");
const generateAnchorElement_1 = require("./transformer/tags/generateAnchorElement");
const getCustomComponents = (page, option) => {
    return {
        a: generateAnchorElement_1.generateAnchorElement(page, option),
    };
};
const getExternalTemplate = (option) => {
    const filename = option.layoutFile;
    if (!filename) {
        return;
    }
    return importer_1.loadExternalFunction(filename);
};
const renderPage = (siteProps, option) => (page) => {
    const createBodyContent = transformer_1.transformRawStringToHtml({
        customComponents: getCustomComponents(page, option),
        props: {},
    });
    const pageProps = {
        site: siteProps,
        article: generateProps_1.generateArticleProps(page),
    };
    const externalTemplate = getExternalTemplate(option);
    const template = createTemplate_1.createTemplate({
        pageProps,
        applyLayout: externalTemplate && externalTemplate.bodyTemplate,
    });
    const bodyContent = createBodyContent(page.content);
    const headContent = transformer_1.createHeadContent(page.metaData);
    return {
        name: path.join(option.serverBasePath, page.name),
        originalName: page.name,
        html: transformer_1.combine({
            head: headContent,
            body: template(bodyContent),
        }),
    };
};
const render = async (source, option) => {
    const siteProps = generateProps_1.generateSiteProps(option);
    return source.pages.map(renderPage(siteProps, option));
};
exports.render = render;
