"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const http = require("http");
const path = require("path");
const url = require("url");
const chokidar = require("chokidar");
const portfinder = require("portfinder");
const mime_types_1 = require("mime-types");
const generator_1 = require("../generator");
const getPage_1 = require("../getPage");
const helpers_1 = require("../helpers");
const reloadScript_1 = require("./reloadScript");
const wsServer_1 = require("./wsServer");
const OBSERVE_FILE_EXTENSION = /\.(js|css|jsx|md|mdx|json)$/;
exports.redirectToLocalFile = (filePath, res) => {
    if (fs.existsSync(filePath) && fs.statSync(filePath).isFile()) {
        res.writeHead(200, { "Content-Type": mime_types_1.lookup(filePath) || "text/plain" });
        fs.createReadStream(filePath).pipe(res);
        return true;
    }
    return false;
};
exports.getRedirectPagePath = (pathname, option) => {
    const calcPath = pathname.endsWith("/") && pathname !== "/" ? pathname.slice(0, pathname.length - 1) : pathname;
    if (option.serverBasePath === "/") {
        return calcPath === "/" ? "/index" : calcPath;
    }
    if (calcPath === "/" || calcPath === option.serverBasePath) {
        return path.join(calcPath, "index");
    }
    return calcPath;
};
exports.getRedirectLocalDirectoryPath = (dirname, pathname, option) => {
    if (pathname.startsWith(option.serverBasePath)) {
        return path.join(dirname, pathname.slice(option.serverBasePath.length));
    }
    return path.join(dirname, pathname);
};
const start = async (dirname, option) => {
    const socketPort = await portfinder.getPortPromise();
    const initialSource = await getPage_1.getData(dirname, option);
    let socket;
    let generatedPages = await generator_1.generateStatic(initialSource, option);
    const watchFiles = [dirname, option.layoutFile || ""];
    const watcher = chokidar.watch(watchFiles, {
        ignoreInitial: true,
    });
    wsServer_1.makeWebSocketServer(socketPort, (res) => {
        socket = res;
    });
    const update = async (updateParams) => {
        if (!socket) {
            return;
        }
        if (path.join(dirname, "rocu.json") === updateParams.filename) {
            const updateConfig = helpers_1.getDefaultConfig(dirname);
            option = { ...option, ...updateConfig };
        }
        const updatedSource = await getPage_1.getData(dirname, { ...option, watcher: updateParams });
        generatedPages = await generator_1.generateStatic(updatedSource, option);
        socket.send(JSON.stringify({ reload: true }));
    };
    watcher.on("change", async (filename) => {
        const base = path.basename(filename);
        const ext = path.extname(base);
        if (!OBSERVE_FILE_EXTENSION.test(ext)) {
            return;
        }
        await update({ filename });
    });
    const app = http.createServer((req, res) => {
        if (!req.url) {
            return;
        }
        const { pathname } = url.parse(req.url);
        if (!pathname) {
            return;
        }
        const filePath = path.join(dirname, pathname);
        if (exports.redirectToLocalFile(filePath, res)) {
            return;
        }
        if (exports.redirectToLocalFile(exports.getRedirectLocalDirectoryPath(dirname, pathname, option), res)) {
            return;
        }
        const name = exports.getRedirectPagePath(pathname, option);
        const renderStaticPage = generatedPages.find((page) => page.name === name);
        if (renderStaticPage) {
            res.write(renderStaticPage.html);
            res.write(reloadScript_1.reloadScript(socketPort));
            res.end();
            return;
        }
        res.write("page not found: " + name);
        res.end();
        return;
    });
    try {
        const server = await app.listen(socketPort + 2);
        return server;
    }
    catch (err) {
        console.error(err);
        throw err;
    }
};
exports.server = start;
