"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const generateLinkElements_1 = require("./tags/generateLinkElements");
const generateScriptElements_1 = require("./tags/generateScriptElements");
exports.generateViewportMetaElements = ({ viewport }) => {
    if (!viewport) {
        return;
    }
    const content = Object.keys(viewport)
        .map(key => `${key}=${viewport[key]}`)
        .join(",");
    return React.createElement("meta", { name: "viewport", content: content });
};
exports.createHeadContent = (htmlMetaData) => {
    return (React.createElement("head", { lang: htmlMetaData.lang || "en" },
        React.createElement("title", null, htmlMetaData.title),
        React.createElement("meta", { key: "charset", charSet: "utf-8" }),
        htmlMetaData.keywords && React.createElement("meta", { name: "keywords", content: htmlMetaData.keywords }),
        htmlMetaData.description && React.createElement("meta", { name: "description", content: htmlMetaData.description }),
        htmlMetaData.copyright && React.createElement("meta", { name: "copyright", content: htmlMetaData.copyright }),
        htmlMetaData.author && React.createElement("meta", { name: "author", content: htmlMetaData.author }),
        htmlMetaData["og:title"] && React.createElement("meta", { property: "og:title", content: htmlMetaData["og:title"] }),
        htmlMetaData["og:description"] && React.createElement("meta", { property: "og:description", content: htmlMetaData["og:description"] }),
        htmlMetaData["og:image"] && React.createElement("meta", { property: "og:image", content: htmlMetaData["og:image"] }),
        htmlMetaData["og:url"] && React.createElement("meta", { property: "og:url", content: htmlMetaData["og:url"] }),
        exports.generateViewportMetaElements(htmlMetaData),
        generateScriptElements_1.generateScriptElements(htmlMetaData),
        generateLinkElements_1.generateLinkElements(htmlMetaData)));
};
